
#app-container .modal-content textarea, #app-container .col-jobs-params pre {
  font-family:monospace;
  background-color:#1c1e22;
  color:white;
  border:1px solid black;
  width:100%;
  font-size:12px;
  overflow:hidden;
  position:relative;
}

#app-container .col-jobs-params pre {
  max-height:300px;
  overflow:scroll;
}

#app-container .modal-content textarea {
  height:500px;
}

table.datatable {
  table-layout: fixed !important;
}
table.datatable td {
  word-wrap:break-word;
}

.container {
  width:inherit;
}

.spin {
  -webkit-animation: spin 1s infinite linear;
  -moz-animation: spin 1s infinite linear;
  -o-animation: spin 1s infinite linear;
  animation: spin 1s infinite linear;
}

@-moz-keyframes spin {
  from {
    -moz-transform: rotate(0deg);
  }
  to {
    -moz-transform: rotate(360deg);
  }
}

@-webkit-keyframes spin {
  from {
    -webkit-transform: rotate(0deg);
  }
  to {
    -webkit-transform: rotate(360deg);
  }
}

@keyframes spin {
  from {
    transform: rotate(0deg);
  }
  to {
    transform: rotate(360deg);
  }
}